import httpx
import jwt
import sentry_sdk
from datetime import (datetime, timedelta)
from enum import (Enum, unique)
from pydantic import (BaseModel)
from urllib.parse import (urljoin)
from typing import (List)


@unique
class IError(Enum):
    OK = 0
    Request_Kong_Error = -60062


class ConsumerModel(BaseModel):
    id: str
    username: str
    custom_id: str
    tags: List[str] = []


class JWTModel(BaseModel):
    key: str
    secret: str
    algorithm: str


class KongAdmin:

    def __init__(self, host):
        self.host = host
        self.max_age = 2600 * 24 * 5

    def get_url(self, path: str) -> str:
        return urljoin(self.host, path)

    async def request(self, method: str, path: str, json: dict = None):
        url = self.get_url(path)
        try:
            async with httpx.AsyncClient() as client:
                r = await client.request(method, url, json=json)

                if r.status_code == 204:
                    return r.status_code, None
                if r.status_code != 200 and r.status_code != 201:
                    return r.status_code, r.json()
                return IError.Ok.value, r.json()
        except httpx.ConnectError as e:
            sentry_sdk.capture_exception(e)
            return IError.Request_Kong_Error.value, None
        except Exception as e:
            sentry_sdk.capture_exception(e)
            return IError.Request_Kong_Error.value, None

    async def get_consumer(self, username) -> (int, ConsumerModel):
        path = f"/consumers/{username}"
        ret, j = await self.request("get", path)
        if ret != IError.Ok.value:
            return ret, None

        return ret, ConsumerModel(**j)

    async def update_or_create_consumer(self, user) -> (int, ConsumerModel):
        path = f"/consumers/{user.userid}"
        custom_id = f"{user.name}/{user.email}"
        data = {
            "username": user.userid,
            "custom_id": custom_id,
            "tags": user.get_tags(),
        }
        ret, j = await self.request("put", path, json=data)
        if ret != IError.Ok.value:
            return ret, None

        return ret, ConsumerModel(**j)

    async def delete_consumer(self, username) -> int:
        path = f"/consumers/{username}"
        err, _ = await self.request("delete", path)
        if err == 204:
            return IError.Ok.value
        return err

    async def list_jwt(self, consumer: str) -> (int, List[JWTModel]):
        path = f"/consumers/{consumer}/jwt"
        ret, j = await self.request("get", path)
        if ret != IError.Ok.value:
            return ret, None

        return ret, [JWTModel(**x) for x in j["data"]]

    async def create_jwt_model(self, username: str) -> (int, JWTModel):
        path = f"/consumers/{username}/jwt"
        ret, j = await self.request("post", path)

        if ret != IError.Ok.value and ret != 201:
            return ret, None

        return IError.Ok.value, JWTModel(**j)

    async def generate_token(self, user):
        await self.update_or_create_consumer(user)

        ret, jwt_models = await self.list_jwt(user.userid)
        if ret != IError.Ok.value:
            return ret, None

        if not jwt_models or len(jwt_models) == 0:
            ret, jwt_model = await self.create_jwt_model(user.userid)
            if ret != IError.Ok.value:
                return ret, None
        else:
            jwt_model = jwt_models[0]

        token = JWT.create_jwt(jwt_model, expires_delta=timedelta(seconds=self.max_age))

        return ret, token


class JWT:
    @staticmethod
    def create_jwt(jwt_model: JWTModel, expires_delta: timedelta = None) -> str:
        if expires_delta:
            expire = datetime.utcnow() + expires_delta
        else:
            expire = datetime.utcnow() + timedelta(seconds=10)
        to_encode = {"iss": jwt_model.key, "exp": expire}
        return jwt.encode(
            to_encode, jwt_model.secret, algorithm=jwt_model.algorithm
        ).decode()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    # 包的分发名称，使用字母、数字、_、-
    name="Rkongcore",
    version="0.0.1",
    # 作者名
    author="ranyu",
    # 作者邮箱
    author_email="wy176404@163.com",
    # 包的简介描述
    description="Libraries for encryption and alignment",
    # 包的详细介绍(一般通过加载README.md)
    long_description=long_description,
    # 和上条命令配合使用，声明加载的是markdown文件
    long_description_content_type="text/markdown",
    # 项目开源地址
    url="",
    # 如果项目由多个文件组成，我们可以使用find_packages()自动发现所有包和子包，而不是手动列出每个包，在这种情况下，包列表将是example_pkg
    packages=setuptools.find_packages(),
    # 关于包的其他元数据(metadata)
    classifiers=[
        # 该软件包仅与Python3兼容
        "Programming Language :: Python :: 3.8",
        # 根据MIT许可证开源
        "License :: OSI Approved :: MIT License",
        # 与操作系统无关
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'httpx~=0.13.3',
        'pydantic~=2.7.1',
        'pydantic_core~=2.18.2',
        'pyjwt~=1.7.1',
        'sentry-sdk~=1.5.12'
    ]
)
